﻿using System;
using System.Collections.Generic;
using System.Data;
using System.Data.SqlClient;
using System.Linq;
using System.Text;
using System.Xml;
using System.Reflection;

namespace MatchData
{
    /// <summary>
    /// find all email through analyse the xml string 'matchdata'
    /// </summary>
    public class AnalyseMatchData
    {
        private readonly string connectionStr;
        private readonly string matchDataSqlStr;
        MatchWithMatchData matchFunction;//use to reflect the xml node to the corresponding function
        Type matchType;//the MatchWithMatchData class 's type
        XmlDocument xmldoc;
        DateTime nowDt;

        public AnalyseMatchData()
        {
            connectionStr = System.Configuration.ConfigurationManager.ConnectionStrings["MatchData"].ConnectionString;
            matchDataSqlStr = "SELECT [Id],[Email],[MatchData]FROM [MatchData].[dbo].[EmailInTime]";
            matchFunction = new MatchWithMatchData();
            matchType = matchFunction.GetType();
            xmldoc = new XmlDocument();
        }
        /// <summary>
        /// the api to get the email address of  match all conditions 
        /// </summary>
        /// <returns></returns>
        public string GetMailAddressByMatchData()
        {
            StringBuilder emailAddresses = new StringBuilder();
            nowDt = DateTime.Now;//Convert.ToDateTime("2013-06-26T22:15:00.000+08:00");//
            using (SqlDataReader dr = SqlHelper.ExecuteReader(connectionStr, CommandType.Text, matchDataSqlStr))
            {
                while (dr.Read())
                {
                    string matchData = dr.GetString(dr.GetOrdinal("MatchData"));
                    if (IsMatchWithMatchData(matchData, nowDt))
                    {
                        string emails = dr.GetString(dr.GetOrdinal("Email"));
                        emailAddresses.Append(string.IsNullOrWhiteSpace(emails) ? string.Empty : ";" + emails);
                    }
                }
            }
            return emailAddresses.Length > 0 ? emailAddresses.Remove(0, 1).ToString() : string.Empty;
        }
        /// <summary>
        /// judge whether the current time match all conditions in the xml string 'matchData'
        /// </summary>
        /// <param name="matchData"></param>
        /// <param name="nowDt"></param>
        /// <returns></returns>
        private bool IsMatchWithMatchData(string matchData, DateTime nowDt)
        {
            bool result = true;
            xmldoc.LoadXml(matchData);
            try
            {
                XmlNodeList xmllist = xmldoc.SelectSingleNode("ScheduleDefinition").ChildNodes;
                DateTime startDt = Convert.ToDateTime(xmllist[0].InnerText).ToLocalTime();
                matchFunction.InitData(startDt, nowDt);
                foreach (XmlNode xmlnode in xmllist)
                {
                    MethodInfo method = matchType.GetMethod(xmlnode.Name + "Function", BindingFlags.Instance | BindingFlags.Public);
                    object oj = method.Invoke(matchFunction, new object[] { xmlnode.OuterXml });
                    if (result)
                        result = result && Convert.ToBoolean(oj);
                    else
                        break;
                }
                return result;
            }
            catch
            {
                
                return false;
            }
        }
    }
}
